class UnrealI_WarlordRocket expands WarlordRocket;

// replicated built-in vectors hold inexact values, float values are replicated precisely
struct ReplicatedVector
{
	var float X, Y, Z;
};

var float InitialSpeed;

var bool bSynchronizeClient;
var ReplicatedVector ReplicatedLocation;
var ReplicatedVector ReplicatedVelocity;
var ReplicatedVector ReplicatedAcceleration;

var UGoldFixSharedInfo SharedInfo;

replication
{
	reliable if (Role == ROLE_Authority)
		InitialSpeed,
		ReplicatedLocation,
		ReplicatedVelocity,
		ReplicatedAcceleration;
}


simulated function Tick(float DeltaTime)
{
	if (!Region.Zone.bWaterZone && VSize(Velocity) < InitialSpeed)
	{
		Velocity = Normal(Velocity) * InitialSpeed;
		Acceleration = vect(0, 0, 0);
	}

	if (bSynchronizeClient)
		Server_UpdateMovement();
	bSynchronizeClient = false;

	if (Level.NetMode == NM_DedicatedServer)
		return;

	Client_UpdateMovement();
	MakeRocketTrail(DeltaTime);
}

simulated function MakeRocketTrail(float DeltaTime)
{
	local SpriteSmokePuff smoke;
	local Bubble1 bubble;
	local float RocketTrailInterval;

	Count += DeltaTime;
	RocketTrailInterval = SmokeRate + FRand() * SmokeRate;
	if (Count >= RocketTrailInterval) 
	{
		if (Region.Zone.bWaterZone)
		{
	 		bubble = Spawn(class'Bubble1',,, Location);
			if (bubble != none)
			{
				bubble.DrawScale = 0.1 + FRand() * 0.2;
				bubble.RemoteRole = ROLE_None;
			}
		}
		else
		{
			smoke = Spawn(class'SpriteSmokePuff');
			if (smoke != none)
				smoke.RemoteRole = ROLE_None;
		}
		Count -= RocketTrailInterval;
		if (Count > RocketTrailInterval)
			Count = RocketTrailInterval;
	}
}

auto state Flying
{
	function ZoneChange(Zoneinfo NewZone)
	{
		local WaterRing w;

		if (VSize(NewZone.ZoneVelocity) != 0)
			bSynchronizeClient = true;

		if (Region.Zone.bWaterZone != NewZone.bWaterZone)
		{
			w = Spawn(class'WaterRing',,,,rot(16384,0,0));
			w.DrawScale = 0.2;
		}
		if (bSynchronizeClient)
			Server_UpdateMovement();
	}

	function HitWall(vector HitNormal, actor Wall)
	{
		if (Mover(Wall) != None && Mover(Wall).bDamageTriggered)
			Wall.TakeDamage(Damage, instigator, Location, MomentumTransfer * Normal(Velocity), '');
		MakeNoise(1.0);
		Explode(Location + ExploWallOut * HitNormal, HitNormal);
	}

	function Explode(vector HitLocation, vector HitNormal)
	{
		local SpriteBallExplosion explosion_effect;
		local UnrealI_WarlordRocket_Explosion explosion_info;

		class'UGoldFix'.static.Projectile_HurtRadius(
			SharedInfo, self, damage, 200.0, 'exploded', MomentumTransfer, HitLocation);

		if (Level.NetMode == NM_DedicatedServer)
		{
			// makes explosion sound on dedicated servers
			explosion_effect = Spawn(class'SpriteBallExplosion',,, HitLocation + HitNormal * 16);
			explosion_effect.RemoteRole = ROLE_None;
		}

		explosion_info = Spawn(class'UnrealI_WarlordRocket_Explosion');
		explosion_info.AssignSourceLocation(Location);
		explosion_info.AssignHitLocation(HitLocation);
		explosion_info.AssignHitNormal(HitNormal);
		explosion_info.SharedInfo = SharedInfo;

		Destroy();
	}

	simulated function BeginState()
	{
		Super.BeginState();
		InitialSpeed = VSize(Velocity);

		if (Level.bHighDetailMode)
			SmokeRate = 0.035;
		else
			SmokeRate = 0.15;
	}
}


function Server_UpdateMovement()
{
	if (Level.NetMode != NM_Standalone)
	{
		ReplicatedLocation = ConvertToReplicatedVector(Location);
		ReplicatedVelocity = ConvertToReplicatedVector(Velocity);
		ReplicatedAcceleration = ConvertToReplicatedVector(Acceleration);
	}
}

simulated function Client_UpdateMovement()
{
	if (Level.NetMode == NM_Client)
	{
		if (VSize(ConvertReplicatedVector(ReplicatedLocation)) != 0)
		{
			SetLocation(ConvertReplicatedVector(ReplicatedLocation));
			ReplicatedLocation = ConvertToReplicatedVector(vect(0, 0, 0));
		}
		if (VSize(ConvertReplicatedVector(ReplicatedVelocity)) != 0)
		{
			Velocity = ConvertReplicatedVector(ReplicatedVelocity);
			Acceleration = ConvertReplicatedVector(ReplicatedAcceleration);
			SetRotation(rotator(Velocity));
			ReplicatedVelocity = ConvertToReplicatedVector(vect(0, 0, 0));
		}
	}
}


static function ReplicatedVector ConvertToReplicatedVector(vector pos)
{
	local ReplicatedVector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

static function vector ConvertReplicatedVector(ReplicatedVector pos)
{
	local vector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

defaultproperties
{
				RemoteRole=ROLE_SimulatedProxy
				bGameRelevant=False
}
